@@ -2,13 +2,15 @@
 # Licensed under the MIT License.
 
 
-class ThresholdOperation():
+class ThresholdOperation:
     """Represents the threshold operations that are used in postprocessing approaches.
 
     Threshold operations simply indicate a threshold and an operator, thereby defining a function.
     The function can be evaluated at arbitrary points (usually the scores returned from
     unconstrained predictors) to return a bool value.
 
+    Read more in the :ref:`User Guide <postprocessing>`.
+
     :param operator: the threshold operator, can be either '>' or '<'
     :type operator: str
     :param threshold: the threshold, can be numpy.inf or -numpy.inf
@@ -16,7 +18,7 @@ class ThresholdOperation():
     """
 
     def __init__(self, operator, threshold):
-        if operator not in ['>', '<']:
+        if operator not in [">", "<"]:
             raise ValueError("Unrecognized operator: " + operator)
         self._operator = operator
         self._threshold = threshold
@@ -39,9 +41,9 @@ class ThresholdOperation():
         :return: the result of elementwise application of the threshold rule
         :rtype: array
         """
-        if self._operator == '>':
+        if self._operator == ">":
             return y_hat > self._threshold
-        elif self._operator == '<':
+        elif self._operator == "<":
             return y_hat < self._threshold
         else:
             raise ValueError("Unrecognized operator: " + self._operator)
