@@ -15,33 +15,31 @@ _SIGN = "sign"
 class Moment:
     """Generic moment.
 
-    Our implementations of the reductions approach to fairness described
-    in `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`_ make use
-    of :class:`Moment` objects to describe the disparity constraints
+    Our implementations of the reductions approach to fairness
+    :footcite:p:`agarwal2018reductions` make use
+    of :class:`Moment` objects to describe both the optimization objective
+    and the fairness constraints
     imposed on the solution. This is an abstract class for all such objects.
+
+    Read more in the :ref:`User Guide <reductions>`.
     """
 
     def __init__(self):
         self.data_loaded = False
 
-    def load_data(self,
-                  X,
-                  y: pd.Series,
-                  *,
-                  sensitive_features: pd.Series = None):
+    def load_data(self, X, y: pd.Series, *, sensitive_features: pd.Series = None):
         """Load a set of data for use by this object.
 
         Parameters
         ----------
         X : array
             The feature array
-        y : pandas.Series
+        y : :class:`pandas.Series`
             The label vector
-        sensitive_features : pandas.Series
+        sensitive_features : :class:`pandas.Series`
             The sensitive feature vector (default None)
         """
-        assert self.data_loaded is False, \
-            "data can be loaded only once"
+        assert self.data_loaded is False, "data can be loaded only once"
         if sensitive_features is not None:
             assert isinstance(sensitive_features, pd.Series)
         self.X = X
@@ -78,6 +76,10 @@ class Moment:
         """Return the signed weights."""
         raise NotImplementedError()
 
+    def _moment_type(self):
+        """Return the moment type, e.g., ClassificationMoment vs LossMoment."""
+        return NotImplementedError()
+
 
 # Ensure that Moment shows up in correct place in documentation
 # when it is used as a base class
@@ -87,6 +89,9 @@ Moment.__module__ = "fairlearn.reductions"
 class ClassificationMoment(Moment):
     """Moment that can be expressed as weighted classification error."""
 
+    def _moment_type(self):
+        return ClassificationMoment
+
 
 # Ensure that ClassificationMoment shows up in correct place in documentation
 # when it is used as a base class
@@ -100,6 +105,9 @@ class LossMoment(Moment):
         super().__init__()
         self.reduction_loss = loss
 
+    def _moment_type(self):
+        return LossMoment
+
 
 # Ensure that LossMoment shows up in correct place in documentation
 # when it is used as a base class
