@@ -1,14 +1,24 @@
 # Copyright (c) Microsoft Corporation and Fairlearn contributors.
 # Licensed under the MIT License.
 
-import pandas as pd
 import numpy as np
+import pandas as pd
+
+from fairlearn.utils._input_validation import (
+    _MESSAGE_RATIO_NOT_IN_RANGE,
+    _validate_and_reformat_input,
+)
 
-from .moment import ClassificationMoment
-from .moment import _GROUP_ID, _LABEL, _PREDICTION, _ALL, _EVENT, _SIGN
-from fairlearn._input_validation import _MESSAGE_RATIO_NOT_IN_RANGE, _validate_and_reformat_input
 from .error_rate import ErrorRate
-
+from .moment import (
+    _ALL,
+    _EVENT,
+    _GROUP_ID,
+    _LABEL,
+    _PREDICTION,
+    _SIGN,
+    ClassificationMoment,
+)
 
 _UPPER_BOUND_DIFF = "upper_bound_diff"
 _LOWER_BOUND_DIFF = "lower_bound_diff"
@@ -47,7 +57,7 @@ class UtilityParity(ClassificationMoment):
     Constraint violation for difference-based constraints starts if the
     difference between a group and the overall population with regard to a
     utility exceeds `difference_bound`. For ratio-based constraints, the ratio
-    between the group-level and overal mean utility needs to be bounded
+    between the group-level and overall mean utility needs to be bounded
     between `ratio_bound` and its inverse (plus an additional additive
     `ratio_bound_slack`).
 
@@ -62,6 +72,8 @@ class UtilityParity(ClassificationMoment):
     - The characters `+` and `-`, corresponding to the Lagrange multipliers
       for positive and negative violations of the constraint
 
+    Read more in the :ref:`User Guide <constraints_binary_classification>`.
+
     Parameters
     ----------
     difference_bound : float
@@ -84,7 +96,9 @@ class UtilityParity(ClassificationMoment):
         Default 0.0
     """
 
-    def __init__(self, *, difference_bound=None, ratio_bound=None, ratio_bound_slack=0.0):
+    def __init__(
+        self, *, difference_bound=None, ratio_bound=None, ratio_bound_slack=0.0
+    ):
         """Initialize with the ratio value."""
         super(UtilityParity, self).__init__()
         if (difference_bound is None) and (ratio_bound is None):
@@ -106,39 +120,48 @@ class UtilityParity(ClassificationMoment):
         """Return the default objective for moments of this kind."""
         return ErrorRate()
 
-    def load_data(self,
-                  X,
-                  y: pd.Series,
-                  *,
-                  sensitive_features: pd.Series,
-                  event: pd.Series = None,
-                  utilities=None):
+    def load_data(
+        self,
+        X,
+        y: pd.Series,
+        *,
+        sensitive_features: pd.Series,
+        event: pd.Series = None,
+        utilities=None,
+    ):
         """Load the specified data into this object.
 
         This adds a column `event` to the `tags` field.
 
-        The `utilities` is a 2-d array which correspond to g(X,A,Y,h(X)) as
-        mentioned in the paper
-        `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`.
+        The `utilities` is a 2-d array which corresponds to g(X,A,Y,h(X))
+        from :footcite:t:`agarwal2018reductions`.
         The `utilities` defaults to h(X), i.e. [0, 1] for each X_i.
         The first column is G^0 and the second is G^1.
         Assumes binary classification with labels 0/1.
 
         .. math::
             utilities = [g(X,A,Y,h(X)=0), g(X,A,Y,h(X)=1)]
+
         """
         super().load_data(X, y, sensitive_features=sensitive_features)
         self.tags[_EVENT] = event
         if utilities is None:
-            utilities = np.vstack([np.zeros(y.shape, dtype=np.float64),
-                                   np.ones(y.shape, dtype=np.float64)]).T
+            utilities = np.vstack(
+                [
+                    np.zeros(y.shape, dtype=np.float64),
+                    np.ones(y.shape, dtype=np.float64),
+                ]
+            ).T
         self.utilities = utilities
         self.prob_event = self.tags.groupby(_EVENT).size() / self.total_samples
-        self.prob_group_event = self.tags.groupby(
-            [_EVENT, _GROUP_ID]).size() / self.total_samples
-        signed = pd.concat([self.prob_group_event, self.prob_group_event],
-                           keys=["+", "-"],
-                           names=[_SIGN, _EVENT, _GROUP_ID])
+        self.prob_group_event = (
+            self.tags.groupby([_EVENT, _GROUP_ID]).size() / self.total_samples
+        )
+        signed = pd.concat(
+            [self.prob_group_event, self.prob_group_event],
+            keys=["+", "-"],
+            names=[_SIGN, _EVENT, _GROUP_ID],
+        )
         self.index = signed.index
         self.default_objective_lambda_vec = None
 
@@ -151,7 +174,7 @@ class UtilityParity(ClassificationMoment):
         # speed up GridSearch.
         self.pos_basis = pd.DataFrame()
         self.neg_basis = pd.DataFrame()
-        self.neg_basis_present = pd.Series(dtype='float64')
+        self.neg_basis_present = pd.Series(dtype="float64")
         zero_vec = pd.Series(0.0, self.index)
         i = 0
         for event_val in event_vals:
@@ -173,26 +196,35 @@ class UtilityParity(ClassificationMoment):
             predictions = np.squeeze(predictions)
         pred = utility_diff.T * predictions + self.utilities[:, 0]
         self.tags[_PREDICTION] = pred
-        expect_event = self.tags.groupby(_EVENT).mean()
-        expect_group_event = self.tags.groupby(
-            [_EVENT, _GROUP_ID]).mean()
-        expect_group_event[_UPPER_BOUND_DIFF] = self.ratio * expect_group_event[_PREDICTION] - \
-            expect_event[_PREDICTION]
-        expect_group_event[_LOWER_BOUND_DIFF] = - expect_group_event[_PREDICTION] \
-            + self.ratio * expect_event[_PREDICTION]
-        g_signed = pd.concat([expect_group_event[_UPPER_BOUND_DIFF],
-                              expect_group_event[_LOWER_BOUND_DIFF]],
-                             keys=["+", "-"],
-                             names=[_SIGN, _EVENT, _GROUP_ID])
-        self._gamma_descr = str(expect_group_event[[_PREDICTION, _UPPER_BOUND_DIFF,
-                                                    _LOWER_BOUND_DIFF]])
+        expect_event = self.tags.groupby(_EVENT).mean(numeric_only=True)
+        expect_group_event = self.tags.groupby([_EVENT, _GROUP_ID]).mean()
+        expect_group_event[_UPPER_BOUND_DIFF] = (
+            self.ratio * expect_group_event[_PREDICTION] - expect_event[_PREDICTION]
+        )
+        expect_group_event[_LOWER_BOUND_DIFF] = (
+            -expect_group_event[_PREDICTION] + self.ratio * expect_event[_PREDICTION]
+        )
+        g_signed = pd.concat(
+            [
+                expect_group_event[_UPPER_BOUND_DIFF],
+                expect_group_event[_LOWER_BOUND_DIFF],
+            ],
+            keys=["+", "-"],
+            names=[_SIGN, _EVENT, _GROUP_ID],
+        )
+        self._gamma_descr = str(
+            expect_group_event[[_PREDICTION, _UPPER_BOUND_DIFF, _LOWER_BOUND_DIFF]]
+        )
         return g_signed
 
     def bound(self):
         """Return bound vector.
 
-        :return: a vector of bound values corresponding to all constraints
-        :rtype: pandas.Series
+        Returns
+        -------
+        pandas.Series
+            a vector of bound values corresponding to all constraints
+
         """
         return pd.Series(self.eps, index=self.index)
 
@@ -208,9 +240,11 @@ class UtilityParity(ClassificationMoment):
             lambda_neg = -lambda_pos
             lambda_pos[lambda_pos < 0.0] = 0.0
             lambda_neg[lambda_neg < 0.0] = 0.0
-            lambda_projected = pd.concat([lambda_pos, lambda_neg],
-                                         keys=["+", "-"],
-                                         names=[_SIGN, _EVENT, _GROUP_ID])
+            lambda_projected = pd.concat(
+                [lambda_pos, lambda_neg],
+                keys=["+", "-"],
+                names=[_SIGN, _EVENT, _GROUP_ID],
+            )
             return lambda_projected
         return lambda_vec
 
@@ -218,7 +252,7 @@ class UtilityParity(ClassificationMoment):
         """Compute the signed weights.
 
         Uses the equations for :math:`C_i^0` and :math:`C_i^1` as defined
-        in Section 3.2 of `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`_
+        in Section 3.2 of :footcite:t:`agarwal2018reductions`
         in the 'best response of the Q-player' subsection to compute the
         signed weights to be applied to the data by the next call to the underlying
         estimator.
@@ -227,14 +261,20 @@ class UtilityParity(ClassificationMoment):
         ----------
         lambda_vec : :class:`pandas:pandas.Series`
             The vector of Lagrange multipliers indexed by `index`
+
         """
-        lambda_event = (lambda_vec["+"] - self.ratio * lambda_vec["-"]).sum(level=_EVENT) / \
-            self.prob_event
-        lambda_group_event = (self.ratio * lambda_vec["+"] - lambda_vec["-"]) / \
-            self.prob_group_event
+        lambda_event = (lambda_vec["+"] - self.ratio * lambda_vec["-"]).groupby(
+            level=_EVENT
+        ).sum() / self.prob_event
+        lambda_group_event = (
+            self.ratio * lambda_vec["+"] - lambda_vec["-"]
+        ) / self.prob_group_event
         adjust = lambda_event - lambda_group_event
         signed_weights = self.tags.apply(
-            lambda row: 0 if pd.isna(row[_EVENT]) else adjust[row[_EVENT], row[_GROUP_ID]], axis=1
+            lambda row: 0
+            if pd.isna(row[_EVENT])
+            else adjust[row[_EVENT], row[_GROUP_ID]],
+            axis=1,
         )
         utility_diff = self.utilities[:, 1] - self.utilities[:, 0]
         signed_weights = utility_diff.T * signed_weights
@@ -259,11 +299,10 @@ class DemographicParity(UtilityParity):
     :class:`pandas:pandas.Series`
     will only have a single entry, which will be equal to 1.
     Similarly, the `index` property will have twice as many entries
-    (corresponding to the Lagrange multipliers for positive and negative constraints)
-    as there are unique values for the sensitive feature.
-    The :meth:`signed_weights` method will compute the costs according
-    to Example 3 of
-    `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`_.
+    (corresponding to the Lagrange multipliers for positive and negative
+    constraints) as there are unique values for the sensitive feature.
+    The :meth:`UtilityParity.signed_weights` method will compute the costs
+    according to Example 3 of :footcite:t:`agarwal2018reductions`.
 
     This :class:`~Moment` also supports control features, which can be used to
     stratify the data, with the Demographic Parity constraint applied within
@@ -272,17 +311,22 @@ class DemographicParity(UtilityParity):
 
     .. math::
       P[h(X) = 1 | A = a, C = c] = P[h(X) = 1 | C = c] \; \forall a, c
+
+    Read more in the :ref:`User Guide <demographic_parity>`.
+
     """
 
     short_name = "DemographicParity"
 
     def load_data(self, X, y, *, sensitive_features, control_features=None):
         """Load the specified data into the object."""
-        _, y_train, sf_train, cf_train = \
-            _validate_and_reformat_input(X, y,
-                                         enforce_binary_labels=True,
-                                         sensitive_features=sensitive_features,
-                                         control_features=control_features)
+        _, y_train, sf_train, cf_train = _validate_and_reformat_input(
+            X,
+            y,
+            enforce_binary_labels=True,
+            sensitive_features=sensitive_features,
+            control_features=control_features,
+        )
 
         base_event = pd.Series(data=_ALL, index=y_train.index)
         event = _merge_event_and_control_columns(base_event, cf_train)
@@ -292,7 +336,7 @@ class DemographicParity(UtilityParity):
 class TruePositiveRateParity(UtilityParity):
     r"""Implementation of true positive rate parity as a moment.
 
-    .. note:
+    .. note::
 
         The true positive rate parity fairness criterion is also known
         as "equal opportunity".
@@ -315,25 +359,30 @@ class TruePositiveRateParity(UtilityParity):
     by two (for the Lagrange multipliers for positive and negative
     constraints).
 
-    With these definitions, the :meth:`signed_weights` method will calculate
-    the costs for `Y=1` as they are calculated in Example 4 of
-    `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`, but will use
+    With these definitions, the :meth:`UtilityParity.signed_weights` method
+    will calculate the costs for `Y=1` as they are calculated in Example 4 of
+    :footcite:t:`agarwal2018reductions`, but will use
     the weights equal to zero for `Y=0`.
 
     This :class:`~Moment` also supports control features, which can be used to
     stratify the data, with the constraint applied within
     each stratum, but not between strata.
+
+    Read more in the :ref:`User Guide <true_positive_rate_parity>`.
+
     """
 
     short_name = "TruePositiveRateParity"
 
     def load_data(self, X, y, *, sensitive_features, control_features=None):
         """Load the specified data into the object."""
-        _, y_train, sf_train, cf_train = \
-            _validate_and_reformat_input(X, y,
-                                         enforce_binary_labels=True,
-                                         sensitive_features=sensitive_features,
-                                         control_features=control_features)
+        _, y_train, sf_train, cf_train = _validate_and_reformat_input(
+            X,
+            y,
+            enforce_binary_labels=True,
+            sensitive_features=sensitive_features,
+            control_features=control_features,
+        )
 
         # The `where` clause is used to put `pd.nan` on all values where `Y!=1`.
         base_event = y_train.apply(lambda v: _LABEL + "=" + str(v)).where(y_train == 1)
@@ -361,25 +410,30 @@ class FalsePositiveRateParity(UtilityParity):
     either NaN or `label=0` (so only one unique non-NaN value), multiplied by
     two (for the Lagrange multipliers for positive and negative constraints).
 
-    With these definitions, the :meth:`signed_weights` method will calculate
+    With these definitions, the :meth:`UtilityParity.signed_weights` method will calculate
     the costs for `Y=0` as they are calculated in Example 4 of
-    `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`,
+    :footcite:t:`agarwal2018reductions`,
     but will use the weights equal to zero for `Y=1`.
 
     This :class:`~Moment` also supports control features, which can be used to
     stratify the data, with the constraint applied within
     each stratum, but not between strata.
+
+    Read more in the :ref:`User Guide <false_positive_rate_parity>`.
+
     """
 
     short_name = "FalsePositiveRateParity"
 
     def load_data(self, X, y, *, sensitive_features, control_features=None):
         """Load the specified data into the object."""
-        _, y_train, sf_train, cf_train = \
-            _validate_and_reformat_input(X, y,
-                                         enforce_binary_labels=True,
-                                         sensitive_features=sensitive_features,
-                                         control_features=control_features)
+        _, y_train, sf_train, cf_train = _validate_and_reformat_input(
+            X,
+            y,
+            enforce_binary_labels=True,
+            sensitive_features=sensitive_features,
+            control_features=control_features,
+        )
 
         # The `where` clause is used to put `pd.nan` on all values where `Y!=0`.
         base_event = y_train.apply(lambda v: _LABEL + "=" + str(v)).where(y_train == 0)
@@ -407,24 +461,29 @@ class EqualizedOdds(UtilityParity):
     the number of unique values of the `Y` array, multiplied by two (for
     the Lagrange multipliers for positive and negative constraints).
 
-    With these definitions, the :meth:`signed_weights` method
+    With these definitions, the :meth:`UtilityParity.signed_weights` method
     will calculate the costs according to Example 4 of
-    `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`_.
+    :footcite:t:`agarwal2018reductions`.
 
     This :class:`~Moment` also supports control features, which can be used to
     stratify the data, with the constraint applied within
     each stratum, but not between strata.
+
+    Read more in the :ref:`User Guide <equalized_odds>`.
+
     """
 
     short_name = "EqualizedOdds"
 
     def load_data(self, X, y, *, sensitive_features, control_features=None):
         """Load the specified data into the object."""
-        _, y_train, sf_train, cf_train = \
-            _validate_and_reformat_input(X, y,
-                                         enforce_binary_labels=True,
-                                         sensitive_features=sensitive_features,
-                                         control_features=control_features)
+        _, y_train, sf_train, cf_train = _validate_and_reformat_input(
+            X,
+            y,
+            enforce_binary_labels=True,
+            sensitive_features=sensitive_features,
+            control_features=control_features,
+        )
 
         base_event = y_train.apply(lambda v: _LABEL + "=" + str(v))
         event = _merge_event_and_control_columns(base_event, cf_train)
@@ -447,29 +506,32 @@ class ErrorRateParity(UtilityParity):
     The `index` property will have twice as many entries (corresponding to the Lagrange multipliers
     for positive and negative constraints) as there are unique values for the sensitive feature.
 
-    The :meth:`signed_weights` method will compute the costs according to Example 3 of
-    `Agarwal et al. (2018) <https://arxiv.org/abs/1803.02453>`_.
+    The :meth:`UtilityParity.signed_weights` method will compute the costs according to Example 3
+    of :footcite:t:`agarwal2018reductions`.
     However, in this scenario, g = abs(h(x)-y), rather than g = h(x)
 
     This :class:`~Moment` also supports control features, which can be used to
     stratify the data, with the constraint applied within
     each stratum, but not between strata.
+
+    Read more in the :ref:`User Guide <error_rate_parity>`.
+
     """
 
     short_name = "ErrorRateParity"
 
     def load_data(self, X, y, *, sensitive_features, control_features=None):
         """Load the specified data into the object."""
-        _, y_train, sf_train, cf_train = \
-            _validate_and_reformat_input(X, y,
-                                         enforce_binary_labels=True,
-                                         sensitive_features=sensitive_features,
-                                         control_features=control_features)
-        utilities = np.vstack([y_train, 1-y_train]).T
+        _, y_train, sf_train, cf_train = _validate_and_reformat_input(
+            X,
+            y,
+            enforce_binary_labels=True,
+            sensitive_features=sensitive_features,
+            control_features=control_features,
+        )
+        utilities = np.vstack([y_train, 1 - y_train]).T
         base_event = pd.Series(data=_ALL, index=y_train.index)
         event = _merge_event_and_control_columns(base_event, cf_train)
         super().load_data(
-            X, y_train,
-            event=event,
-            utilities=utilities,
-            sensitive_features=sf_train)
+            X, y_train, event=event, utilities=utilities, sensitive_features=sf_train
+        )
